@@ -30,6 +30,11 @@ session <- local({
     stopifnot(dir.create(execdir));
     setwd(execdir);
     
+    #copy files to execdir
+    lapply(req$files(), function(x){
+      stopifnot(file.copy(x$tmp_name, x$name))
+    });
+    
     #setup handler
     myhandler <- evaluate::new_output_handler(value=function(myval){
       if(isTRUE(storeval)){
@@ -85,28 +90,39 @@ session <- local({
     saveRDS(output, file=".REval");
     saveRDS(sessionInfo(), file=".RInfo");  
     saveRDS(.libPaths(), file=".Rlibs"); 
-    
-    #store results permanently
-    hash <- generate();
-    
+
     #does not work on windows 
     #stopifnot(file.rename(execdir, sessiondir(hash))); 
     
-    stopifnot(dir.create(sessiondir(hash), recursive=TRUE))
-    stoponwarn(file.copy(list.files(recursive=TRUE, all=TRUE), sessiondir(hash)))
+    #store results permanently
+    hash <- generate();  
+    outputdir <- sessiondir(hash);
     
-    #send output format. Default is to send a list.
-    switch(format,
-      "json" = sendjson(hash, get(".val", sessionenv)),
-      sendlist(hash)    
-    );
+    #First try renaming to destionation directory
+    if(!isTRUE(file.rename(execdir, outputdir))){
+      #When rename fails, try copying instead
+      suppressWarnings(dir.create(dirname(outputdir)));
+      stopifnot(file.copy(execdir, dirname(outputdir), recursive=TRUE));
+      setwd(dirname(outputdir));
+      stopifnot(file.rename(basename(execdir), basename(outputdir)));
+      unlink(execdir, recursive=TRUE);      
+    }
+
+    #Shortcuts to get object immediately
+    if(format %in% c("json", "print")){
+      sendobject(hash, get(".val", sessionenv), format);
+    } else {
+      #default: send 201 with output list.
+      sendlist(hash)
+    }
   }
   
-  sendjson <- function(hash, obj){
+  sendobject <- function(hash, obj, format){
     tmppath <- sessionpath(hash);
     outputpath <- paste(req$mount(), tmppath, "/", sep="");    
-    res$setheader("Location", outputpath);    
-    httpget_object(obj, "json");
+    res$setheader("Location", outputpath); 
+    res$setheader("X-ocpu-session", hash)
+    httpget_object(obj, format);
   }
   
   #redirects the client to the session location
@@ -122,6 +138,7 @@ session <- local({
     res$setbody(text);
     res$setheader("Content-Type", 'text/plain; charset="UTF-8"');
     res$setheader("Location", outputpath);
+    res$setheader("X-ocpu-session", hash)
     res$finish(201);    
   }
   
